package com.planet_ink.coffee_mud.Abilities.Druid;
import com.planet_ink.coffee_mud.core.interfaces.*;
import com.planet_ink.coffee_mud.core.*;
import com.planet_ink.coffee_mud.Abilities.interfaces.*;
import com.planet_ink.coffee_mud.Areas.interfaces.*;
import com.planet_ink.coffee_mud.Behaviors.interfaces.*;
import com.planet_ink.coffee_mud.CharClasses.interfaces.*;
import com.planet_ink.coffee_mud.Commands.interfaces.*;
import com.planet_ink.coffee_mud.Common.interfaces.*;
import com.planet_ink.coffee_mud.Exits.interfaces.*;
import com.planet_ink.coffee_mud.Items.interfaces.*;
import com.planet_ink.coffee_mud.Locales.interfaces.*;
import com.planet_ink.coffee_mud.MOBS.interfaces.*;
import com.planet_ink.coffee_mud.Races.interfaces.*;


import java.util.*;

/* 
   Copyright 2000-2006 Bo Zimmerman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

public class Chant_CaveIn extends Chant
{
	public String ID() { return "Chant_CaveIn"; }
	public String name(){ return "Cave-In";}
	public int abstractQuality(){return Ability.QUALITY_MALICIOUS;}
	protected int canAffectCode(){return 0;}
	protected int canTargetCode(){return Ability.CAN_MOBS|CAN_EXITS;}
	public int amountRemaining=0;
	public long flags(){return Ability.FLAG_PARALYZING;}

	public boolean okMessage(Environmental host, CMMsg msg)
	{
		if((affected instanceof Exit)
		&&((msg.targetMinor()==CMMsg.TYP_ENTER)||(msg.targetMinor()==CMMsg.TYP_LEAVE)||(msg.targetMinor()==CMMsg.TYP_OPEN))
		&&(msg.source().envStats().height()>=0)
		&&((msg.tool()==affected)||(msg.target()==affected)))
		{
			msg.source().tell("This exit is blocked by rubble, and can not be moved through.");
			return false;
		}
		else
		if((affected instanceof MOB)
		&&(msg.amISource((MOB)affected)))
		{
			MOB mob=(MOB)affected;
			if(msg.sourceMinor()==CMMsg.TYP_STAND)
				return false;
			if((!CMath.bset(msg.sourceMajor(),CMMsg.MASK_ALWAYS))
			&&((CMath.bset(msg.sourceMajor(),CMMsg.MASK_HANDS))
			||(CMath.bset(msg.sourceMajor(),CMMsg.MASK_MOVE))))
			{
				mob.location().show(mob,null,null,CMMsg.MSG_OK_ACTION,"<S-NAME> struggle(s) to get out from under the rocks.");
				amountRemaining-=(mob.charStats().getStat(CharStats.STAT_STRENGTH)*4);
				if(amountRemaining<0)
					unInvoke();
				else
					return false;
			}
		}
		return super.okMessage(host,msg);
	}

	public void affectEnvStats(Environmental affected, EnvStats affectableStats)
	{
		super.affectEnvStats(affected,affectableStats);
		if(affected instanceof MOB)
		{
			affectableStats.setDisposition(affectableStats.disposition()|EnvStats.IS_BOUND);
			affectableStats.setDisposition(affectableStats.disposition()|EnvStats.IS_SITTING);
		}
	}

	public boolean invoke(MOB mob, Vector commands, Environmental givenTarget, boolean auto, int asLevel)
	{
		Environmental target=null;
		if((commands.size()>0)&&(givenTarget==null))
		{
			int dir=Directions.getGoodDirectionCode(CMParms.combine(commands,0));
			if((dir>=0)&&(dir!=Directions.UP)&&(mob.location().getExitInDir(dir)!=null))
				target=mob.location().getExitInDir(dir);
		}
		if(target==null)
			target=getAnyTarget(mob,commands,givenTarget,Item.WORNREQ_ANY);
		if(target==null) return false;
		if((target instanceof Item)||(target instanceof Room))
		{
			mob.tell("This chant can only target exits or creatures.");
			return false;
		}
		if((!auto)
		&&(mob.location().domainType()!=Room.DOMAIN_INDOORS_CAVE))
		{
			mob.tell("This chant only works in caves.");
			return false;
		}

		if(!super.invoke(mob,commands,givenTarget,auto,asLevel))
			return false;

		boolean success=proficiencyCheck(mob,0,auto);
		if(success)
		{
			// it worked, so build a copy of this ability,
			// and add it to the affects list of the
			// affected MOB.  Then tell everyone else
			// what happened.
			CMMsg msg=CMClass.getMsg(mob,target,this,verbalCastCode(mob,target,auto)|CMMsg.MASK_MALICIOUS,auto?"":"^S<S-NAME> chant(s) at <T-NAMESELF>!^?");
			if(mob.location().okMessage(mob,msg))
			{
				mob.location().send(mob,msg);
				amountRemaining=200;
				if(target instanceof Exit)
					mob.location().show(mob,target,CMMsg.MSG_OK_VISUAL,"A cave-in causes rubble to fall, blocking <T-NAME>!");
				else
				if(target instanceof MOB)
				{
					mob.location().show(mob,target,CMMsg.MSG_OK_VISUAL,"A cave-in drops rocks on <T-NAME>!");
					int maxDie =  (int)Math.round(new Integer(adjustedLevel(mob,asLevel)).doubleValue()/3.0);
					int damage = CMLib.dice().roll(maxDie,3,maxDie);
					if(msg.value()>0)
						damage = (int)Math.round(CMath.div(damage,1.5));
					if(((MOB)target).location()==mob.location())
						CMLib.combat().postDamage(mob,(MOB)target,this,damage,CMMsg.MSG_OK_VISUAL,Weapon.TYPE_BASHING,"The falling rubble <DAMAGE> <T-NAME>!");
				}
				if(msg.value()<=0)
					success=maliciousAffect(mob,target,asLevel,(target instanceof Exit)?0:10,0);
			}
		}
		else
			return maliciousFizzle(mob,target,"<S-NAME> chant(s) at <T-NAMESELF>, but nothing happens.");


		// return whether it worked
		return success;
	}
}
